-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

M.type = "auxiliary"

local FileSize = 89168

local customcetloaded = nil
local customisloaded = ""
local radiatorFanRPM = 0
local radiatorFanRPMSmooth = 0
local FanRPMSmooth = false
local FanRPMSmoothOff = false
local ClutchEngaged = false
local radiatorClutchFanSpin = 0
local mechanicalRadiatorClutchFanRPMCoef = 0
local clutchIdleSpeedCoef = 0.7
local clutchSpeedCoef = 3
local radiatorFanType = nil
local manualEngaged = false
local mainEngine = powertrain.getDevice("mainEngine") or -1

--Get file size from Game Engine then execute global function when received
obj:queueGameEngineLua("be:getPlayerVehicle(0):queueLuaCommand('vFCcETFS(' .. getFileSize(\"lua/vehicle/powertrain/combustionEngineThermals.lua\") .. ')')")

local function init(jbeamData)

  electrics.values.vFCData = {}
  electrics.values['radiatorClutchFanSpin'] = 0

  if mainEngine ~= -1 then
    mechanicalRadiatorClutchFanRPMCoef = 1 / (mainEngine.torqueData.maxRPM or 8000 * 0.7)
    customcetloaded = v.data.mainEngine.thermalsLuaFileName or nil
    radiatorFanType = v.data.mainEngine.radiatorFanType or nil
    clutchIdleSpeedCoef = jbeamData.clutchIdleSpeedCoef or 0.7
    clutchSpeedCoef = jbeamData.clutchSpeedCoef or 3
    --print(customcetloaded or "nil")
    --print(radiatorFanType or "nil")
  end
end

function vFCcETFS(cETFS)
  if customcetloaded == "powertrain/vFCcET" and radiatorFanType == "viscousfanclutch" then
    customisloaded = " (custom combustionEngineThermals.lua (vFCcET.lua) is loaded)"
  else
    customisloaded = " (custom combustionEngineThermals.lua (vFCcET.lua) is not loaded)"
  end

  if cETFS ~= -1 then
    if cETFS == FileSize then
      log("D", "viscousFanClutch.lua", string.format("viscousFanClutch.lua has been initialised and is working correctly" .. customisloaded))
    else
      log("W", "viscousFanClutch.lua", "Initialised but combustionEngineThermals.lua has changed in size things may not work correctly")
      log("D", "viscousFanClutch.lua", "file size has changed from " .. FileSize .. " to " .. cETFS .. customisloaded)
    end
  else
    log("E", "viscousFanClutch.lua", "Initialised but error getting file size of combustionEngineThermals.lua things may not work correctly (cETFS is " .. cETFS .. ")")
    log("D", "viscousFanClutch.lua", "" .. customisloaded)
  end
--print(cETFS or "nil")
end

M.init = init

if mainEngine == -1 then return M end

--local vFCMgr = require("extensions/vFCMgr")

local max = math.max
local min = math.min
local abs = math.abs

local clutchFanAirSpeed = 0
local radiatorFanMaxAirSpeed = 0

local clutchfanidleSmoother = newTemporalSmoothing(200, 200)
local clutchfanSmoother = newTemporalSmoothing(5000, 5000)
local radiatorFanRPMSmoother = 0
local radiatorFanRPMidleSmoother = 0
local FanRPMSmoothRate = 0.05

local radiatorFanMaxAirSpeed = v.data.mainEngine.radiatorFanMaxAirSpeed or 4
local radiatorFanTemperature = v.data.mainEngine.radiatorFanTemperature or 110

--print(radiatorFanTemperature or "nil")


local function updateGFX(dt)

local engineRPM = electrics.values.rpm
local coolantTemperature = mainEngine.thermals.coolantTemperature
local thermostatTemperature = mainEngine.thermals.debugData.engineThermalData.thermostatTemperature

--receive data from combustionEngineThermals.lua

--sends data to combustionEngineThermals.lua
--electrics.values['clutchFanAirSpeed'] = clutchFanAirSpeed

  --mechanical clutch fans are either engaged or disengaged, depending on coolant temperature
  if coolantTemperature >= radiatorFanTemperature or manualEngaged then
  ClutchEngaged = true
  elseif coolantTemperature <= thermostatTemperature * 1.05 and not manualEngaged then
  ClutchEngaged = false
  end

if ClutchEngaged == true then
  if FanRPMSmooth == false and FanRPMSmoothOff == false then
    FanRPMSmoothOff = true
    FanRPMSmooth = true
  end
  FanRPMSmoothRate = 0.05
  radiatorFanRPMSmooth = radiatorFanRPMSmoother
  elseif ClutchEngaged == false then
    if FanRPMSmooth == false and FanRPMSmoothOff == true then
    FanRPMSmoothOff = false
    FanRPMSmooth = true
  end
  FanRPMSmoothRate = 0.007
  radiatorFanRPMSmooth = radiatorFanRPMidleSmoother
  end

if FanRPMSmooth == true or engineRPM >= 10 then
radiatorFanRPM = radiatorFanRPM + (radiatorFanRPMSmooth-radiatorFanRPM)*FanRPMSmoothRate
else
radiatorFanRPM = radiatorFanRPMSmooth
end

if abs(radiatorFanRPMSmooth-radiatorFanRPM) <= 1 then
FanRPMSmooth = false
end

radiatorFanRPMSmoother = clutchfanSmoother:getUncapped(((-clutchSpeedCoef * 10) + engineRPM), dt)
radiatorFanRPMidleSmoother = clutchfanidleSmoother:getUncapped((engineRPM * -clutchIdleSpeedCoef + engineRPM), dt)

electrics.values.vFCData['clutchFanAirSpeed'] = radiatorFanMaxAirSpeed * min(max(radiatorFanRPM * mechanicalRadiatorClutchFanRPMCoef, 0), 1)

electrics.values['radiatorClutchFanSpin'] = (electrics.values['radiatorClutchFanSpin'] + radiatorFanRPM * dt) % 360

--print(clutchFanAirSpeed or "nil")
--print(FanRPMSmooth or "false")
--print(FanRPMSmoothRate or "nil")

  electrics.values.vFCData.radiatorFanRPM = radiatorFanRPM
  electrics.values.vFCData.FanRPMSmooth = FanRPMSmooth
  electrics.values.vFCData.ClutchEngaged = ClutchEngaged
end

local function reset(jbeamData)
  electrics.values.vFCData = {}
  ClutchEngaged = false
  radiatorClutchFanSpin = 0
  radiatorFanRPM = 0
  clutchfanidleSmoother:reset()
  clutchfanSmoother:reset()
  radiatorFanRPMSmooth = 0
  FanRPMSmoothOff = false
  FanRPMSmooth = false
end

local function manualEngage(engage)
  manualEngaged = engage
end

M.reset = reset
M.manualEngage = manualEngage
M.updateGFX = updateGFX

return M